#include "extern.h"


void Copia (STRUCTURE *Destino, STRUCTURE Origen)
/* Copia un individuo en otro */

{
 int i;

 (*Destino).fitness=Origen.fitness;
 for (i=0;i<n_genes;i++)
  (*Destino).Gene[i]=Origen.Gene[i];
 }


int compara (const void *a, const void *b)
/* Funcion para comparar dos valores empleada por el algoritmo qsort, el cual
   se encarga de ordenar la poblacion genetica de mejor a peor fitness.
   Depende de la macro BETTER, que se encuentra en define.h, la cual indica
   si el problema es de maximizacion o minimizacion */

{
 if (BETTER ((* (STRUCTURE *) a).fitness, (* (STRUCTURE *) b).fitness))
  return (-1);
 else
  return (1);
 }



void Seleccion_Baker ()
/* Seleccion basada en el esquema de asignacion de probabilidades
   de orden lineal y la ruleta de Baker. */

{
 float rank_min, rank_max, salto, u;
 int i, j, temp;
 
 rank_min=0.75;
 rank_max=2.0-rank_min;

 /* Ordenamos la poblacion de menor a mayor rango */
 qsort ((void *) Old,long_poblacion,sizeof (STRUCTURE),compara);
 
 /* Calculamos la probabilidad de seleccion de cada individuo mediante
    el ranking lineal en funcion de su posicion en el orden */
 for (i=0;i<long_poblacion;i++)
  Old[i].n_e=(rank_max-(rank_max-rank_min)*i/(float)(long_poblacion-1))/(float)long_poblacion;

 /* Se procede a la seleccion de los cromosomas segun el metodo de Baker */
 Ruleta[0]=Old[0].n_e;
 for (i=1;i<long_poblacion;i++)
  Ruleta[i] = Ruleta[i-1] + Old[i].n_e;
 
 u=Rand (); salto=1/(float)long_poblacion; i=0;
 for (j=0;j<long_poblacion;j++)
  {
   for (;Ruleta[i]<u;i++);
   sample[j]=i;
   u+=salto;
   if (u>1.0)
    {
     u-=1.0;
     i=0;
     }
   }
  
 /* Se procede a barajar los cromosomas seleccionados para aplicar posterior-
    mente los operadores geneticos */
 for (i=0;i<long_poblacion;i++)
  {
   j=Randint (i,long_poblacion-1);
   temp=sample[j];
   sample[j]=sample[i];
   sample[i]=temp;
   }

 /* Se crea la nueva poblacion a partir de ese baraje  */
 for (i=0;i<long_poblacion;i++)
  {
   Copia(&New[i],Old[sample[i]]);
   New[i].n_e=0;
   }
 }
 
void Seleccion_Torneo (void)
/* Selecciona la poblacion intermedia por la tecnica del Torneo */

{
 int i, j, k, repetido, rango_act, inicio_grupo, num_elem_grupo, mejor_torneo;
 float suma;
 
 /* Ordenamos la poblacion por fitness */
 qsort ((void *) Old,long_poblacion,sizeof (STRUCTURE),compara);
 
 
 /* Se procede a la seleccion de los cromosomas segun el metodo del Torneo */
 for (i=0;i<long_poblacion;i++)
  {
   Torneo[0] = Randint(0,long_poblacion-1);
   mejor_torneo=Torneo[0];
   
   for (j=1;j<tam_torneo;j++)
    {
     do
      {
       Torneo[j] = Randint(0,long_poblacion-1);
       repetido=0; k=0;
       while ((k<j) && (!repetido))
        if (Torneo[j]==Torneo[k])
         repetido=1;
        else
         k++;
       }
     while (repetido);
     
     if (Torneo[j]<mejor_torneo)
      mejor_torneo=Torneo[j];
     }
       
   sample[i] = mejor_torneo;
   }

 /* Se crea la nueva poblacion */
 for (i=0;i<long_poblacion;i++)
  Copia(&New[i],Old[sample[i]]);
 }


